@@ -10,21 +10,26 @@
 #  License for the specific language governing permissions and limitations
 #  under the License.
 
-"""Application base class.
-"""
+"""Application base class."""
 
+import argparse
 import inspect
 import locale
 import logging
 import logging.handlers
 import os
 import sys
+import typing as ty
 
 from cliff import _argparse
 from . import complete
 from . import help
 from . import utils
 
+if ty.TYPE_CHECKING:
+    from . import command as _command
+    from . import commandmanager as _commandmanager
+    from . import interactive as _interactive
 
 logging.getLogger('cliff').addHandler(logging.NullHandler())
 
@@ -34,7 +39,7 @@ _SIGINT_EXIT = 130
 _SIGPIPE_EXIT = 141
 
 
-class App(object):
+class App:
     """Application base class.
 
     :param description: one-liner explaining the program purpose
@@ -63,17 +68,26 @@ class App(object):
     LOG = logging.getLogger(NAME)
 
     CONSOLE_MESSAGE_FORMAT = '%(message)s'
-    LOG_FILE_MESSAGE_FORMAT = \
+    LOG_FILE_MESSAGE_FORMAT = (
         '[%(asctime)s] %(levelname)-8s %(name)s %(message)s'
+    )
     DEFAULT_VERBOSE_LEVEL = 1
     DEFAULT_OUTPUT_ENCODING = 'utf-8'
 
-    def __init__(self, description, version, command_manager,
-                 stdin=None, stdout=None, stderr=None,
-                 interactive_app_factory=None,
-                 deferred_help=False):
-        """Initialize the application.
-        """
+    def __init__(
+        self,
+        description: ty.Optional[str],
+        version: ty.Optional[str],
+        command_manager: '_commandmanager.CommandManager',
+        stdin: ty.Optional[ty.TextIO] = None,
+        stdout: ty.Optional[ty.TextIO] = None,
+        stderr: ty.Optional[ty.TextIO] = None,
+        interactive_app_factory: ty.Optional[
+            type['_interactive.InteractiveApp']
+        ] = None,
+        deferred_help: bool = False,
+    ) -> None:
+        """Initialize the application."""
         self.command_manager = command_manager
         self.command_manager.add_command('help', help.HelpCommand)
         self.command_manager.add_command('complete', complete.CompleteCommand)
@@ -82,9 +96,14 @@ class App(object):
         self.deferred_help = deferred_help
         self.parser = self.build_option_parser(description, version)
         self.interactive_mode = False
-        self.interpreter = None
+        self.interpreter: ty.Optional[_interactive.InteractiveApp] = None
 
-    def _set_streams(self, stdin, stdout, stderr):
+    def _set_streams(
+        self,
+        stdin: ty.Optional[ty.TextIO],
+        stdout: ty.Optional[ty.TextIO],
+        stderr: ty.Optional[ty.TextIO],
+    ) -> None:
         try:
             locale.setlocale(locale.LC_ALL, '')
         except locale.Error:
@@ -120,8 +139,12 @@ class App(object):
         self.stdout = stdout or sys.stdout
         self.stderr = stderr or sys.stderr
 
-    def build_option_parser(self, description, version,
-                            argparse_kwargs=None):
+    def build_option_parser(
+        self,
+        description: ty.Optional[str],
+        version: ty.Optional[str],
+        argparse_kwargs: ty.Optional[dict[str, ty.Any]] = None,
+    ) -> _argparse.ArgumentParser:
         """Return an argparse option parser for this application.
 
         Subclasses may override this method to extend
@@ -137,25 +160,25 @@ class App(object):
         """
         argparse_kwargs = argparse_kwargs or {}
         parser = _argparse.ArgumentParser(
-            description=description,
-            add_help=False,
-            **argparse_kwargs
+            description=description, add_help=False, **argparse_kwargs
         )
         parser.add_argument(
             '--version',
             action='version',
-            version='{0} {1}'.format(App.NAME, version),
+            version=f'{App.NAME} {version}',
         )
         verbose_group = parser.add_mutually_exclusive_group()
         verbose_group.add_argument(
-            '-v', '--verbose',
+            '-v',
+            '--verbose',
             action='count',
             dest='verbose_level',
             default=self.DEFAULT_VERBOSE_LEVEL,
             help='Increase verbosity of output. Can be repeated.',
         )
         verbose_group.add_argument(
-            '-q', '--quiet',
+            '-q',
+            '--quiet',
             action='store_const',
             dest='verbose_level',
             const=0,
@@ -169,14 +192,16 @@ class App(object):
         )
         if self.deferred_help:
             parser.add_argument(
-                '-h', '--help',
+                '-h',
+                '--help',
                 dest='deferred_help',
                 action='store_true',
                 help="Show help message and exit.",
             )
         else:
             parser.add_argument(
-                '-h', '--help',
+                '-h',
+                '--help',
                 action=help.HelpAction,
                 nargs=0,
                 default=self,  # tricky
@@ -190,9 +215,8 @@ class App(object):
         )
         return parser
 
-    def configure_logging(self):
-        """Create logging handlers for any log output.
-        """
+    def configure_logging(self) -> None:
+        """Create logging handlers for any log output."""
         root_logger = logging.getLogger('')
         root_logger.setLevel(logging.DEBUG)
 
@@ -207,17 +231,18 @@ class App(object):
 
         # Always send higher-level messages to the console via stderr
         console = logging.StreamHandler(self.stderr)
-        console_level = {0: logging.WARNING,
-                         1: logging.INFO,
-                         2: logging.DEBUG,
-                         }.get(self.options.verbose_level, logging.DEBUG)
+        console_level = {
+            0: logging.WARNING,
+            1: logging.INFO,
+            2: logging.DEBUG,
+        }.get(self.options.verbose_level, logging.DEBUG)
         console.setLevel(console_level)
         formatter = logging.Formatter(self.CONSOLE_MESSAGE_FORMAT)
         console.setFormatter(formatter)
         root_logger.addHandler(console)
         return
 
-    def print_help_if_requested(self):
+    def print_help_if_requested(self) -> None:
         """Print help and exits if deferred help is enabled and requested.
 
         '--help' shows the help message and exits:
@@ -227,10 +252,10 @@ class App(object):
            explicitly this method in initialize_app.
         """
         if self.deferred_help and self.options.deferred_help:
-            action = help.HelpAction(None, None, default=self)
+            action = help.HelpAction([], argparse.SUPPRESS, default=self)
             action(self.parser, self.options, None, None)
 
-    def run(self, argv):
+    def run(self, argv: list[str]) -> int:
         """Equivalent to the main program for the application.
 
         :param argv: input arguments and options
@@ -272,7 +297,7 @@ class App(object):
             return _SIGINT_EXIT
         result = 1
         if self.interactive_mode:
-            result = self.interact()
+            self.interact()
         else:
             try:
                 result = self.run_subcommand(remainder)
@@ -284,7 +309,7 @@ class App(object):
 
     # FIXME(dhellmann): Consider moving these command handling methods
     # to a separate class.
-    def initialize_app(self, argv):
+    def initialize_app(self, argv: list[str]) -> None:
         """Hook for subclasses to take global initialization action
         after the arguments are parsed but before a command is run.
         Invoked only once, even in interactive mode.
@@ -294,7 +319,7 @@ class App(object):
         """
         return
 
-    def prepare_to_run_command(self, cmd):
+    def prepare_to_run_command(self, cmd: '_command.Command') -> None:
         """Perform any preliminary work needed to run a command.
 
         :param cmd: command processor being invoked
@@ -302,7 +327,12 @@ class App(object):
         """
         return
 
-    def clean_up(self, cmd, result, err):
+    def clean_up(
+        self,
+        cmd: '_command.Command',
+        result: int,
+        err: ty.Optional[BaseException],
+    ) -> None:
         """Hook run after a command is done to shutdown the app.
 
         :param cmd: command processor being invoked
@@ -314,22 +344,25 @@ class App(object):
         """
         return
 
-    def interact(self):
+    def interact(self) -> None:
         # Defer importing .interactive as cmd2 is a slow import
         from .interactive import InteractiveApp
 
         if self.interactive_app_factory is None:
-            self.interactive_app_factory = InteractiveApp
-        self.interpreter = self.interactive_app_factory(self,
-                                                        self.command_manager,
-                                                        self.stdin,
-                                                        self.stdout,
-                                                        )
-        return self.interpreter.cmdloop()
+            interactive_app_factory = InteractiveApp
+            self.interactive_app_factory = interactive_app_factory
+        else:
+            interactive_app_factory = self.interactive_app_factory
+        self.interpreter = interactive_app_factory(
+            self,
+            self.command_manager,
+            self.stdin,
+            self.stdout,
+        )
+        self.interpreter.cmdloop()
 
-    def get_fuzzy_matches(self, cmd):
-        """return fuzzy matches of unknown command
-        """
+    def get_fuzzy_matches(self, cmd: str) -> list[str]:
+        """return fuzzy matches of unknown command"""
 
         sep = '_'
         if self.command_manager.convert_underscores:
@@ -343,8 +376,12 @@ class App(object):
                 dist.append((0, candidate))
                 continue
             # Levenshtein distance
-            dist.append((utils.damerau_levenshtein(cmd, prefix, utils.COST)+1,
-                         candidate))
+            dist.append(
+                (
+                    utils.damerau_levenshtein(cmd, prefix, utils.COST) + 1,
+                    candidate,
+                )
+            )
 
         matches = []
         match_distance = 0
@@ -360,10 +397,10 @@ class App(object):
 
         return matches
 
-    def run_subcommand(self, argv):
+    def run_subcommand(self, argv: list[str]) -> int:
         try:
             subcommand = self.command_manager.find_command(argv)
-        except ValueError as err:
+        except ValueError as exc:
             # If there was no exact match, try to find a fuzzy match
             the_cmd = argv[0]
             fuzzy_matches = self.get_fuzzy_matches(the_cmd)
@@ -371,32 +408,40 @@ class App(object):
                 article = 'a'
                 if self.NAME[0] in 'aeiou':
                     article = 'an'
-                self.stdout.write('%s: \'%s\' is not %s %s command. '
-                                  'See \'%s --help\'.\n'
-                                  % (self.NAME, ' '.join(argv), article,
-                                      self.NAME, self.NAME))
+                self.stdout.write(
+                    '{}: \'{}\' is not {} {} command. '
+                    'See \'{} --help\'.\n'.format(
+                        self.NAME,
+                        ' '.join(argv),
+                        article,
+                        self.NAME,
+                        self.NAME,
+                    )
+                )
                 self.stdout.write('Did you mean one of these?\n')
                 for match in fuzzy_matches:
-                    self.stdout.write('  %s\n' % match)
+                    self.stdout.write(f'  {match}\n')
             else:
                 if self.options.debug:
                     raise
                 else:
-                    self.LOG.error(err)
+                    self.LOG.error(exc)
             return 2
+
         cmd_factory, cmd_name, sub_argv = subcommand
         kwargs = {}
         if 'cmd_name' in inspect.getfullargspec(cmd_factory.__init__).args:
             kwargs['cmd_name'] = cmd_name
         cmd = cmd_factory(self, self.options, **kwargs)
         result = 1
-        err = None
+        err: ty.Optional[BaseException] = None
         try:
             self.prepare_to_run_command(cmd)
-            full_name = (cmd_name
-                         if self.interactive_mode
-                         else ' '.join([self.NAME, cmd_name])
-                         )
+            full_name = (
+                cmd_name
+                if self.interactive_mode
+                else ' '.join([self.NAME, cmd_name])
+            )
             cmd_parser = cmd.get_parser(full_name)
             try:
                 parsed_args = cmd_parser.parse_args(sub_argv)
@@ -404,6 +449,7 @@ class App(object):
                 if self.interactive_mode:
                     # Defer importing cmd2 as it is a slow import
                     import cmd2
+
                     raise cmd2.exceptions.Cmd2ArgparseError from ex
                 else:
                     raise ex
